/** @file
File:       IccMpeXml.h

Contains:   Header for implementation of CIccMpeXml class and
creation factories

Version:    V1

Copyright:   see ICC Software License
*/

/*
 * The ICC Software License, Version 0.2
 *
 *
 * Copyright (c) 2003-2010 The International Color Consortium. All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. In the absence of prior written permission, the names "ICC" and "The
 *    International Color Consortium" must not be used to imply that the
 *    ICC organization endorses or promotes products derived from this
 *    software.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE INTERNATIONAL COLOR CONSORTIUM OR
 * ITS CONTRIBUTING MEMBERS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the The International Color Consortium. 
 *
 *
 * Membership in the ICC is encouraged when this software is used for
 * commercial purposes. 
 *
 *  
 * For more information on The International Color Consortium, please
 * see <http://www.color.org/>.
 *  
 * 
 */


#ifndef _ICCMPEXML_H
#define _ICCMPEXML_H
#include "IccMpeBasic.h"
#include "IccMpeACS.h"
#include <libxml/parser.h>
#include <libxml/tree.h>

class CIccMpeXml : public IIccExtensionMpe
{
public:
  virtual ~CIccMpeXml(void) {}

  virtual bool ToXml(std::string &xml, std::string blanks="") = 0;
  virtual bool ParseXml(xmlNode *pNode, std::string &parseStr) = 0;

  virtual const char *GetExtClassName() const { return "CIccMpeXml"; }
};

class CIccMpeXmlUnknown : public CIccMpeUnknown, public CIccMpeXml
{
public:
  virtual ~CIccMpeXmlUnknown() {}

  virtual const char *GetClassName() const { return "CIccTagXmlUnknown"; }

  virtual IIccExtensionMpe *GetExtension() { return this; }

  virtual bool ToXml(std::string &xml, std::string blanks="");
  virtual bool ParseXml(xmlNode *pNode, std::string &parseStr);
};

class CIccMpeXmlCurveSet : public CIccMpeCurveSet, public CIccMpeXml
{
public:
  virtual ~CIccMpeXmlCurveSet() {}

  virtual const char *GetClassName() const { return "CIccMpeXmlCurveSet"; }

  virtual IIccExtensionMpe *GetExtension() { return this; }

  virtual bool ToXml(std::string &xml, std::string blanks="");
  virtual bool ParseXml(xmlNode *pNode, std::string &parseStr);

protected:
  //virtual CIccCurveSetCurve* CreateCurve(icCurveElemSignature sig);
};

class CIccMpeXmlMatrix : public CIccMpeMatrix, public CIccMpeXml
{
public:
  virtual ~CIccMpeXmlMatrix() {}

  virtual const char *GetClassName() const { return "CIccMpeXmlMatrix"; }

  virtual IIccExtensionMpe *GetExtension() { return this; }

  virtual bool ToXml(std::string &xml, std::string blanks="");
  virtual bool ParseXml(xmlNode *pNode, std::string &parseStr);
};

class CIccMpeXmlCLUT : public CIccMpeCLUT, public CIccMpeXml
{
public:
  virtual ~CIccMpeXmlCLUT() {}

  virtual const char *GetClassName() const { return "CIccMpeXmlCLUT"; }

  virtual IIccExtensionMpe *GetExtension() { return this; }

  virtual bool ToXml(std::string &xml, std::string blanks="");
  virtual bool ParseXml(xmlNode *pNode, std::string &parseStr);
};

class CIccMpeXmlBAcs : public CIccMpeBAcs, public CIccMpeXml
{
public:
  virtual ~CIccMpeXmlBAcs() {}

  virtual const char *GetClassName() const { return "CIccMpeXmlBAcs"; }

  virtual IIccExtensionMpe *GetExtension() { return this; }

  virtual bool ToXml(std::string &xml, std::string blanks="");
  virtual bool ParseXml(xmlNode *pNode, std::string &parseStr);
};

class CIccMpeXmlEAcs : public CIccMpeEAcs, public CIccMpeXml
{
public:
  virtual ~CIccMpeXmlEAcs() {}

  virtual const char *GetClassName() const { return "CIccMpeXmlEAcs"; }

  virtual IIccExtensionMpe *GetExtension() { return this; }

  virtual bool ToXml(std::string &xml, std::string blanks="");
  virtual bool ParseXml(xmlNode *pNode, std::string &parseStr);
};

#endif //_ICCMPEXML_H
